﻿using System;
using System.Collections.Generic;
using System.Text;

namespace InABox.Core
{
    /// <summary>
    /// Base interface for all "Posters" - classes that define the means of processing an <see cref="IPostable"/> <see cref="Entity"/>.
    /// There is little point in directly implementing this; rather, one should implement a subinterface.
    /// The functionality of any <see cref="IPoster{TEntity,TSettings}"/> is essentially the default functionality, and, based on the subinterface implemented,
    /// will be scriptable.
    /// </summary>
    /// <typeparam name="TEntity">The type of entity that this poster can process.</typeparam>
    /// <typeparam name="TSettings">The <see cref="PosterSettings"/> specific to this type of poster.</typeparam>
    public interface IPoster<TEntity, TSettings>
        where TEntity : Entity, IPostable, IRemotable, IPersistent, new()
        where TSettings : PosterSettings
    {
        TSettings Settings { get; set; }
    }

    public interface IGlobalSettingsPoster
    {
        public IGlobalPosterSettings GlobalSettings { get; set; }
    }

    public interface IGlobalSettingsPoster<TGlobalSettings> : IGlobalSettingsPoster
        where TGlobalSettings : BaseObject, IGlobalPosterSettings
    {
        public new TGlobalSettings GlobalSettings { get; set; }

        IGlobalPosterSettings IGlobalSettingsPoster.GlobalSettings
        {
            get => GlobalSettings;
            set
            {
                if(value is TGlobalSettings settings)
                {
                    GlobalSettings = settings;
                }
            }
        }
    }
}
